;================================================================================
// PWM samples at 15720Hz, with <7 bits, sample values in range[0,109]
;================================================================================
PlaySamplePWM_target_start = $400
;================================================================================

// if we need it faster, the main code of this could live in page 0..

	.proc PlaySamplePWM, PlaySamplePWM_target_start
	pha				; save return bank

// according to "de re atari": Next, channel 1's volume is set to zero. Since channel 1 no longer has its own direct output,
// its volume setting is meaningless to us and we zero it. Channel 1's frequency register is used as the fine or low byte
// in the sound generation, and channel 2's frequency register is the coarse or high byte.

// STIMER: You POKE any value here to load and start the timers; the value isn't itself used in the calculations.
// This resets all of the audio frequency dividers to their AUDF values.

Init
	lda #%11000000
	sta GRAFP0
	lda #$1E
	sta COLPM0
	
	lda #255
	sta AUDF2			// channel 2, high byte always = 255

    lda #$AF			// pure tone (distortion 10), max vol (15), for channel 1
    sta AUDC1
	
    lda #$50			// Clock channel one with 1.79 MHz + Join channels two and one (16 bit)
    sta AUDCTL
	
    ldy #>$C000			// end of cart buffer window (+1)

FirstBankSMAddress
	ldx #SAMPLE1_FIRST_BANK		// first bank for the sample, that needs to be in consecutive banks
	sta $d500,x
	
	sta WSYNC
	sta WSYNC
	
PlayLoop
    lda SampleCartBuffer		// values in the range 0 - 109
	
    sta WSYNC			// only one for sample rate 15720 in NTSC (15600 in PAL)
	
    sta AUDF1			// change channel 1 (low byte)
    sta STIMER
	
	// asl			// assuming range with values lower than 128..
	// sta COLBK
	
	clc
	adc #[128-55]
	sta HPOSP0
	
    inc PlayLoop+1
    bne PlayLoop
    inc PlayLoop+2
	
    cpy PlayLoop+2
	
    bne PlayLoop
	
    lda #>SampleCartBuffer		// reset pointer for next bank
    sta PlayLoop+2
	
	inx
	sta $d500,x

LastBankSMAddress
	cpx #[SAMPLE1_FIRST_BANK + SAMPLE1_CART_BANKS]
	bne PlayLoop

ExitPlayLoop
	lda #$18
	sta AUDC1
	
	lda #0
	sta GRAFP0

	pla
	tax
	sta $d500,x		; enable return bank
	
	rts
	.endp
	
PlaySamplePWM_size = .len PlaySamplePWM
PlaySamplePWM_cart_start = .adr PlaySamplePWM

