;================================================================================
// PWM samples at 21013Hz, with >6 bits, sample values in range[0,80]
;================================================================================
PlaySamplePWM4_target_start = $800
;================================================================================

	.proc PlaySamplePWM4, PlaySamplePWM4_target_start
	pha				; save return bank

Init
	lda #255
	sta AUDF2			// channel 2, high byte always = 255
    lda #$AF			// pure tone (distortion 10), max vol (15), for channel 1
    sta AUDC1
    lda #$50			// Clock channel one with 1.79 MHz + Join channels two and one (16 bit)
    sta AUDCTL
	
	lda #32				// number of pages for 8k
	sta m_bankPages

	lda #0
	sta m_samplePointer
	lda #>SampleCartBuffer
	sta m_samplePointer+1

	ldy #0
	
FirstBankSMAddress
	ldx #SAMPLE1_FIRST_BANK		// first bank for the sample, that needs to be in consecutive banks
	sta $d500,x

	sta WSYNC
	sta WSYNC

// final version: values in the range [0, 80], 15760 * 4/3 = 21013.33 Hz, 114 * 3 / 4 = 85.5 (86-85-86-85 cycles)
// playing groups of 4 samples let us pair the worst case (updating the pointers, crossing pages, changing banks) with the use of wsync

// if wsync is position 0.0, then next is 0.75, then 1.5, then 2.25..
// from 0 to 0.75 next line: 9 refresh cycles [86 - 9 = 77 cycles]
// from 0.75 to 0.5 next line: 9 refresh cycles [85 - 7 = 78 cycles]
// from 0.5 to 0.25 next line: 0 refresh cycles [85 cycles]

PlayLoop
    lda (m_samplePointer),y
	
    sta WSYNC
	
    sta AUDF1
    sta STIMER
	
    iny			// 2 cycles
	
	// we need 77 (+9 refresh) [86] cycles between both "sta stimer", so here we need 62 cycles
	jsr Burn56Cycles		// 6 cycles
	
    lda (m_samplePointer),y		// 5 cycles
    sta AUDF1					// 4 cycles
    sta STIMER					// 4 cycles
	
	iny
	
	// we need 78 (+7 refresh) [85] cycles between both "sta stimer", so here we need 63 cycles
	jsr Burn57Cycles		// 6 cycles
	// stimer has a +1 delay here
	
    lda (m_samplePointer),y		// 5 cycles
    sta AUDF1					// 4 cycles
    sta STIMER					// 4 cycles
	
	iny
	
	// we need 85 [85] cycles between both "sta stimer", so here we need 70 cycles
	jsr Burn64Cycles			// 6 cycles
	// stimer has a +1 delay here
	
    lda (m_samplePointer),y		// 5 cycles
    sta AUDF1					// 4 cycles
    sta STIMER					// 4 cycles
	
	iny

	
    bne PlayLoop
	
    inc m_samplePointer+1
	
    dec m_bankPages
    bne PlayLoop
	
	lda #32				// number of pages for 8k
	sta m_bankPages
	
    lda #>SampleCartBuffer		// reset pointer for next bank
    sta m_samplePointer+1
	
	inx
	sta $d500,x

LastBankSMAddress
	cpx #[SAMPLE1_FIRST_BANK + SAMPLE1_CART_BANKS]
	bne PlayLoop

ExitPlayLoop
	lda #$18
	sta AUDC1
	
	pla
	tax
	sta $d500,x		; enable return bank
	
	rts


Burn57Cycles
	tya			// 2 cycles
	ldy #8		// 2 cycles
	
B57Loop			// for the loop there is (5*8 - 1) = 39
	dey
	bne B57Loop

	nop
	nop
	nop

	tay			// 2 cycles
	
	rts			// 6 cycles
	
	
Burn56Cycles
	tya			// 2 cycles
	ldy #9		// 2 cycles
	
B56Loop			// for the loop there is (5*9 - 1) = 44
	dey
	bne B56Loop

	tay			// 2 cycles
	
	rts			// 6 cycles
	
	
Burn64Cycles
	tya			// 2 cycles
	ldy #10		// 2 cycles
	
B64Loop			// for the loop there is (5*10 - 1) = 49
	dey
	bne B64Loop

	beq B64Here		// 3 cycles
B64Here

	tay			// 2 cycles
	
	rts			// 6 cycles
	
	.endp
	
PlaySamplePWM4_size = .len PlaySamplePWM4
PlaySamplePWM4_cart_start = .adr PlaySamplePWM4

