;================================================================================
// PWM samples at 31440Hz, with <6 bits, sample values in range[0,52]
;================================================================================
PlaySamplePWM5_target_start = $900
;================================================================================

	.proc PlaySamplePWM5, PlaySamplePWM5_target_start
	pha				; save return bank

Init
	lda #255
	sta AUDF2			// channel 2, high byte always = 255
    lda #$AF			// pure tone (distortion 10), max vol (15), for channel 1
    sta AUDC1
    lda #$50			// Clock channel one with 1.79 MHz + Join channels two and one (16 bit)
    sta AUDCTL
	
	lda #32				// number of pages for 8k
	sta m_bankPages

	lda #0
	sta m_samplePointer
	lda #>SampleCartBuffer
	sta m_samplePointer+1

	ldy #0
	
FirstBankSMAddress
	ldx #SAMPLE1_FIRST_BANK		// first bank for the sample, that needs to be in consecutive banks
	sta $d500,x

	sta WSYNC
	sta WSYNC

// playing groups of 2 samples let us pair the worst case (updating the pointers, crossing pages, changing banks) with the use of wsync

PlayLoop
    lda (m_samplePointer),y		// 5 cycles (+1 if page boundary crossed)
	
    sta WSYNC			// 4 cycles
	
	:4 nop		// 8 cycles later (to avoid the extra separation between the write to audf1 and stimer, due to the refresh cycles)
	
    sta AUDF1			// 4 cycles
    sta STIMER			// 4 cycles

// use this instead of the writes to audf1 and stimer, to be able to see the position of the changes in the screen
	// stx COLBK
	// sta COLBK

//----------------------------------------
// this section needs to add to 57 cycles, and we need to account for the 9 refresh cycles, so we need 48 cycles in instructions
    iny			// 2 cycles
	
	// then this call should expend 48-15 = 33 cycles.. already using 6..
	//jsr DummyTime		// 6 cycles
	
	lda (m_samplePointer),y		// 5 cycles
	sta m_samplePointBuffer		// 3 cycles
	
	cpy #255	// is it going to be 0?
	bne _shortShortBranch
	
	lda m_bankPages
	cmp #1		// is it going to be 0?
	beq _longBranch
	
	bne _shortBranch
	
_longBranch		// 8+4+5+3 + 5+7 + X + 3 ==> X = 35-35 = 0 (no need to burn any code)
	lda #[>SampleCartBuffer]-1		// reset pointer for next bank (-1 because is going to be increased below)
    sta m_samplePointer+1
	
	inx
	sta $d500,x
	
	jmp _exitBranch
	
_shortBranch	// 8+4+5+2+3 + Y ==> Y = 35-22 = 13
	lda (0,x)	// 6 cycles
	nop
	nop
	jmp _exitBranch		// 3 cycles

_shortShortBranch		// 8+5 + Z ==> Z = 35-13 = 22
	lda (0,x)	// 6 cycles
	lda (0,x)	// 6 cycles
	lda (0,x)	// 6 cycles
	nop
	nop

_exitBranch
	lda m_samplePointBuffer	// recovered +2 cycles using this (needs to burn 35 now)
    //lda (m_samplePointer),y		// 5 cycles
	
    sta AUDF1					// 4 cycles
    sta STIMER					// 4 cycles

// use this instead of the writes to audf1 and stimer, to be able to see the position of the changes in the screnn
	// stx COLBK
	// sta COLBK
	
//----------------------------------------
// before, this section worst case was: 4+5+7+5+5+7+5(loop taken) = 38 (+25 from the top = 63!! .. should be lower or equal to 57)
	iny
    bne PlayLoop
	
    inc m_samplePointer+1
	
    dec m_bankPages
    bne PlayLoop

	lda #32				// number of pages for 8k
	sta m_bankPages
	
// so we moved this critical path code to the previous zone used to burn cycles.. (and changed the first value to: #[>SampleCartBuffer]-1)
// ..could move more code, but this is good enough
/*	
    lda #>SampleCartBuffer		// reset pointer for next bank
    sta m_samplePointer+1
	
	inx
	sta $d500,x
*/
LastBankSMAddress
	cpx #[SAMPLE1_FIRST_BANK + SAMPLE1_CART_BANKS]
	bne PlayLoop

ExitPlayLoop
	lda #$18
	sta AUDC1
	
	pla
	tax
	sta $d500,x		; enable return bank
	
	rts

	.endp
	
PlaySamplePWM5_size = .len PlaySamplePWM5
PlaySamplePWM5_cart_start = .adr PlaySamplePWM5

