;	For 1 MB Maxflash with 128 banks of 8k each.
;	Cartmapper "42 - MaxFlash 1M / 8Mbit -older (bank127)
;	or "MaxFlash 1M / 8Mbit - new (bank0)" in Altirra.

;CARTCS	= $bffa 			;Start address vector, used if CARTFG has CARTFG_START_CART bit set
;CART	= $bffc				;Flag, must be zero for modules
;CARTFG	= $bffd				;Flags or-ed together, indicating how to start the module.
;CARTAD	= $bffe				;Initialization address vector

CARTFG_DIAGNOSTIC_CART = $80		;Flag value: Directly jump via CARTAD during RESET.
CARTFG_START_CART = $04			;Flag value: Jump via CARTAD and then via CARTCS.
CARTFG_BOOT = $01			;Flag value: Boot peripherals, then start the module.

;================================================================================
	opt h-				;Disable Atari COM/XEX file headers
;================================================================================

;--------------------------------------------------------------------------------
; includes:

	icl "../base/sys_equates.m65"

	icl "../base/sys_macros.m65"

;--------------------------------------------------------------------------------
; memory map

; remember that some move/copy macros use the last 6 bytes of page zero!

Vars_address			= 0
Macros_vars_address		= 250

;--------------------------------------------------------------------------------
; variables

	org Vars_address
	
m_samplePointer			org *+2
m_bankPages				org *+1
m_samplePointBuffer		org *+1

;--------------------------------------------------------------------------------
; constants

//PAL_VERSION			= 1

;================================================================================================================================================================
;	Bank 0 is the startup bank on newer Atarimax Maxflash 1MB cartridges only.
;	See bank 127 for the compatibility code for the older models.

	org $a000			; Bank 1
	;opt f+				; Activate fill mode

	.proc bank0			; Powerup code for new Atarimax 1 MB cartridges

	.proc init			; Cartridge initalization
	rts					; Only the minimum of the OS initialization is complete, you don't want to code here normally.
	.endp

;--------------------------------------------------------------------------------
	.proc start			; Cartridge start, RAM, graphics 0 and IOCB no for the editor (E:) are ready

	ClearSystem

	DisableBasic
	DisableOperativeSystem
	
	CopyMemory ClearAllMemory_cart_start, ClearAllMemory_target_start, ClearAllMemory_size
	
	lda #0		; return bank
	jsr ClearAllMemory
	
; only run in the correct system
	/*lda PAL
	and #14

.if .def PAL_VERSION
NO_PAL_loop
	bne NO_PAL_loop
.else
NO_NTSC_loop
	beq NO_NTSC_loop
.endif*/

	CopyMemory PlaySamplePWM_cart_start, PlaySamplePWM_target_start, PlaySamplePWM_size
	CopyMemory PlaySamplePWM2_cart_start, PlaySamplePWM2_target_start, PlaySamplePWM2_size
	CopyMemory PlaySamplePCM_cart_start, PlaySamplePCM_target_start, PlaySamplePCM_size
	CopyMemory PlaySamplePWM3_cart_start, PlaySamplePWM3_target_start, PlaySamplePWM3_size
	CopyMemory PlaySamplePWM4_cart_start, PlaySamplePWM4_target_start, PlaySamplePWM4_size
	CopyMemory PlaySamplePWM5_cart_start, PlaySamplePWM5_target_start, PlaySamplePWM5_size

	CopyMemory DisplayList, DisplayListRam, [DisplayListEnd-DisplayList]
	CopyMemory DisplayListData, DisplayListDataRam, [DisplayListDataEnd-DisplayListData]
	CopyMemory AtariFont, AtariFontRam, 1024

	
	VcountSync 1
	
	lda #14
	sta COLPF1
	lda #2
	sta COLPF2
	
	SetFontAddress AtariFontRam
	
	SetDisplayListAddress DisplayListRam

	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL


wait_loop
	lda VCOUNT
	sta WSYNC
	sta COLBK
	
	lda SKCTL
	and #4			// a key pressed
	bne wait_loop
	
	lda KBCODE
	cmp #31				// key "1"
	beq play_sample1
	cmp #30				// key "2"
	beq play_sample2
	cmp #26				// key "3"
	beq play_sample3
	cmp #24				// key "4"
	beq play_sample4
	cmp #29				// key "5"
	beq play_sample5
	
	jmp wait_loop

	
play_sample1
	lda #0
	sta DMACTL
	sta COLBK
	
	lda #SAMPLE1_FIRST_BANK
	sta PlaySamplePWM.FirstBankSMAddress+1
	lda #[SAMPLE1_FIRST_BANK + SAMPLE1_CART_BANKS]
	sta PlaySamplePWM.LastBankSMAddress+1
	
	lda #0		// return bank
	jsr PlaySamplePWM_target_start
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop


play_sample2
	lda #0
	sta DMACTL
	sta COLBK
	
	lda #SAMPLE2_FIRST_BANK
	sta PlaySamplePWM4.FirstBankSMAddress+1
	lda #[SAMPLE2_FIRST_BANK + SAMPLE2_CART_BANKS]
	sta PlaySamplePWM4.LastBankSMAddress+1

	lda #0		// return bank
	jsr PlaySamplePWM4_target_start
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop

	
play_sample3
	lda #0
	sta DMACTL
	sta COLBK
	
	lda #SAMPLE3_FIRST_BANK
	sta PlaySamplePWM5.FirstBankSMAddress+1
	lda #[SAMPLE3_FIRST_BANK + SAMPLE3_CART_BANKS]
	sta PlaySamplePWM5.LastBankSMAddress+1

	lda #0		// return bank
	jsr PlaySamplePWM5_target_start
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop

	
play_sample4
	lda #0
	sta DMACTL
	sta COLBK
	
	lda #SAMPLE4_FIRST_BANK
	sta PlaySamplePCM.FirstBankSMAddress+1
	lda #[SAMPLE4_FIRST_BANK + SAMPLE4_CART_BANKS]
	sta PlaySamplePCM.LastBankSMAddress+1

	lda #0		// return bank
	jsr PlaySamplePCM_target_start
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop

	
play_sample5
	lda #0
	sta DMACTL
	sta COLBK
	
	lda #SAMPLE5_FIRST_BANK
	sta PlaySamplePWM.FirstBankSMAddress+1
	lda #[SAMPLE5_FIRST_BANK + SAMPLE5_CART_BANKS]
	sta PlaySamplePWM.LastBankSMAddress+1

	lda #0		// return bank
	jsr PlaySamplePWM_target_start
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop

	
	.endp

;================================================================================
ClearAllMemory_target_start = $300

	.proc ClearAllMemory, ClearAllMemory_target_start
	pha				; save return bank
	
	sta $d580		; disable all cart banks (enable normal ram)
	
	SetMemory (1*1024) (51*1024) 0
	SetMemory (54*1024) (10*1024) 0

	pla
	tax
	sta $d500,x		; enable return bank
	
	rts
	.endp
	
ClearAllMemory_size = .len ClearAllMemory
ClearAllMemory_cart_start = .adr ClearAllMemory

;================================================================================
	icl "PWM_15K.asm"

	icl "PWM_7K.asm"

	icl "PCM_15K.asm"

	icl "PWM_15K_half_sync.asm"

	icl "PWM_21K.asm"

	icl "PWM_31K.asm"
	
;================================================================================
DisplayListRam = $E00
DisplayList
	:5 .byte DL_BLANK_8
	
	.byte GM_CHAR_G0+DL_LMS_MASK
	.word DisplayListDataRam
	.byte DL_BLANK_8
	.byte GM_CHAR_G0
	.byte DL_BLANK_8
	.byte GM_CHAR_G0
	.byte DL_BLANK_8
	.byte DL_BLANK_4
	.byte GM_CHAR_G0
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte GM_CHAR_G0
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte GM_CHAR_G0
	
	.byte DL_JVB
	.word DisplayListRam
DisplayListEnd

DisplayListDataRam = $F00
DisplayListData
	.sb " 1 - Song 1, PWM -7bits 15720Hz "
	.sb " 2 - Song 1, PWM +6bits 21013Hz "
	.sb " 3 - Song 1, PWM -6bits 31440Hz "
	.sb " 4 - Song 1, PCM  4bits 15720Hz "
	.sb " 5 - Song 2, PWM -7bits 15720Hz "
	.sb "  Press a # to play that song.  "
DisplayListDataEnd

AtariFontRam = $1000
AtariFont
	ins "data\atari.fnt"
	
;================================================================================
	.if * > $bff4
		.error "First bank overwrite final data!"
	.endif
;================================================================================
	.print "------------------------------------------------------------"
	.print "Bank 0 area: [", bank0, ", ", *-1, "]"
	.print " "
;================================================================================

	opt f+				; Activate fill mode
	org $bff4			; Powerup code for old Atarimax 1 MB cartridges

	.proc powerup
	lda $d500			; Dummy, will never be executed, see bank 127
	jmp start			; Continuation from bank 127				
	.endp

	org $bffa			; Cartridge control block
	
	.word start			; CARTCS
	.byte 0				; CART
	.byte CARTFG_START_CART		; CARTFG
	.word init			; CARTAD
	
	.endp

	opt f-				; Deactivate fill mode, so ORG can be changed again

;================================================================================================================================================================
SampleCartBuffer = $A000

SAMPLE1_FIRST_BANK = 1
SAMPLE1_CART_BANKS = 10
	ins "data/t2s_cut_y_15k_pwm.raw"

SAMPLE2_FIRST_BANK = 11
SAMPLE2_CART_BANKS = 13
	ins "data/t2s_cut_y_21k_pwm.raw"
	
SAMPLE3_FIRST_BANK = 24
SAMPLE3_CART_BANKS = 19
	ins "data/t2s_cut_y_31k_pwm.raw"
	
SAMPLE4_FIRST_BANK = 43
SAMPLE4_CART_BANKS = 10
	ins "data/t2s_cut_y_15k_pcm.raw"
	
SAMPLE5_FIRST_BANK = 53
SAMPLE5_CART_BANKS = 67
	ins "data/r9mt2_cut_x_pwm.raw"

;================================================================================================================================================================
;	Including a ready to use binary file with "INS", or part thereof will be much faster

DUMMY_BANKS = (128 - 2) - (SAMPLE1_CART_BANKS + SAMPLE2_CART_BANKS + SAMPLE3_CART_BANKS + SAMPLE4_CART_BANKS + SAMPLE5_CART_BANKS)

	.proc other_banks
	.rept DUMMY_BANKS

	org $a000
	opt f+

	.sav 8192
	
	opt f-

	.endr
	.endp
	
;================================================================================================================================================================
;	Special startup on bank 127 because older Atarimax Maxflash 1MB cartridges have start bank 127
;	Newer cartridges have start bank 0, but you should keep this code for compatibility with older hardware.

	org $a000			; Bank 127
	opt f+
	
	.proc bank_127		; Powerup code for old Atarimax 1 MB cartridges
	
	org $bff4

	.proc powerup
	lda $d500			; Activate bank 0
	jmp *				; Dummy, will never be executed, see bank 0
	.endp

	org $bffa			; Cartridge control block
	
	.word powerup                   ; CARTCS
	.byte 0                         ; CART
	.byte CARTFG_START_CART         ; CARTFG
	.word powerup                   ; CARTAD

	.endp
	
	opt f-

